@@ -78,6 +78,60 @@ public:
 	HANDLE read_{INVALID_HANDLE_VALUE};
 	HANDLE write_{INVALID_HANDLE_VALUE};
 };
+
+native_string escape_argument(native_string const& arg)
+{
+	native_string ret;
+
+	// Treat newlines as whitespace just to be sure, even if MSDN doesn't mention it
+	if (arg.find_first_of(fzT(" \"\t\r\n\v")) != native_string::npos) {
+		// Quite horrible, as per MSDN: 
+		// Backslashes are interpreted literally, unless they immediately precede a double quotation mark.
+		// If an even number of backslashes is followed by a double quotation mark, one backslash is placed in the argv array for every pair of backslashes, and the double quotation mark is interpreted as a string delimiter.
+		// If an odd number of backslashes is followed by a double quotation mark, one backslash is placed in the argv array for every pair of backslashes, and the double quotation mark is "escaped" by the remaining backslash, causing a literal double quotation mark (") to be placed in argv.
+
+		ret = fzT("\"");
+		int backslashCount = 0;
+		for (auto it = arg.begin(); it != arg.end(); ++it) {
+			if (*it == '\\') {
+				++backslashCount;
+			}
+			else {
+				if (*it == '"') {
+					// Escape all preceeding backslashes and escape the quote
+					ret += native_string(backslashCount + 1, '\\');
+				}
+				backslashCount = 0;
+			}
+			ret += *it;
+		}
+		if (backslashCount) {
+			// Escape all preceeding backslashes
+			ret += native_string(backslashCount, '\\');
+		}
+
+		ret += fzT("\"");
+	}
+	else {
+		ret = arg;
+	}
+
+	return ret;
+}
+
+native_string get_cmd_line(native_string const& cmd, std::vector<native_string>::const_iterator const& begin, std::vector<native_string>::const_iterator const& end)
+{
+	native_string cmdline = escape_argument(cmd);
+
+	for (auto it = begin; it != end; ++it) {
+		auto const& arg = *it;
+		if (!arg.empty()) {
+			cmdline += fzT(" ") + escape_argument(arg);
+		}
+	}
+
+	return cmdline;
+}
 }
 
 class process::impl
@@ -100,7 +154,7 @@ public:
 			err_.create(true);
 	}
 
-	bool spawn(native_string const& cmd, std::vector<native_string> const& args)
+	bool spawn(native_string const& cmd, std::vector<native_string>::const_iterator const& begin, std::vector<native_string>::const_iterator const& end)
 	{
 		if (process_ != INVALID_HANDLE_VALUE) {
 			return false;
@@ -117,11 +171,11 @@ public:
 		si.hStdOutput = out_.write_;
 		si.hStdError = err_.write_;
 
-		auto cmdline = get_cmd_line(cmd, args);
+		auto cmdline = get_cmd_line(cmd, begin, end);
 
 		PROCESS_INFORMATION pi{};
 
-		auto cmdline_buf = &cmdline[0];
+		auto cmdline_buf = cmdline.data();
 
 		DWORD const flags = CREATE_UNICODE_ENVIRONMENT | CREATE_DEFAULT_ERROR_MODE | CREATE_NO_WINDOW;
 		BOOL res = CreateProcess(cmd.c_str(), cmdline_buf, nullptr, nullptr, TRUE, flags, nullptr, nullptr, &si, &pi);
@@ -184,59 +238,6 @@ public:
 	}
 
 private:
-	native_string escape_argument(native_string const& arg)
-	{
-		native_string ret;
-
-		// Treat newlines are whitespace just to be sure, even if MSDN doesn't mention it
-		if (arg.find_first_of(fzT(" \"\t\r\n\v")) != native_string::npos) {
-			// Quite horrible, as per MSDN: 
-			// Backslashes are interpreted literally, unless they immediately precede a double quotation mark.
-			// If an even number of backslashes is followed by a double quotation mark, one backslash is placed in the argv array for every pair of backslashes, and the double quotation mark is interpreted as a string delimiter.
-			// If an odd number of backslashes is followed by a double quotation mark, one backslash is placed in the argv array for every pair of backslashes, and the double quotation mark is "escaped" by the remaining backslash, causing a literal double quotation mark (") to be placed in argv.
-
-			ret = fzT("\"");
-			int backslashCount = 0;
-			for (auto it = arg.begin(); it != arg.end(); ++it) {
-				if (*it == '\\') {
-					++backslashCount;
-				}
-				else {
-					if (*it == '"') {
-						// Escape all preceeding backslashes and escape the quote
-						ret += native_string(backslashCount + 1, '\\');
-					}
-					backslashCount = 0;
-				}
-				ret += *it;
-			}
-			if (backslashCount) {
-				// Escape all preceeding backslashes
-				ret += native_string(backslashCount, '\\');
-			}
-
-			ret += fzT("\"");
-		}
-		else {
-			ret = arg;
-		}
-
-		return ret;
-	}
-
-	native_string get_cmd_line(native_string const& cmd, std::vector<native_string> const& args)
-	{
-		native_string cmdline = escape_argument(cmd);
-
-		for (auto const& arg : args) {
-			if (!arg.empty()) {
-				cmdline += fzT(" ") + escape_argument(arg);
-			}
-		}
-
-		return cmdline;
-	}
-
 	HANDLE process_{INVALID_HANDLE_VALUE};
 
 	pipe in_;
@@ -309,6 +310,30 @@ public:
 	int read_{-1};
 	int write_{-1};
 };
+
+void make_arg(native_string const& arg, std::vector<std::unique_ptr<native_string::value_type[]>> & argList)
+{
+	std::unique_ptr<char[]> ret;
+	ret.reset(new char[arg.size() + 1]);
+	memcpy(ret.get(), arg.c_str(), arg.size() + 1);
+	argList.push_back(std::move(ret));
+}
+
+void get_argv(native_string const& cmd, std::vector<native_string>::const_iterator const& begin, std::vector<native_string>::const_iterator const& end, std::vector<std::unique_ptr<char[]>> & argList, std::unique_ptr<char *[]> & argV)
+{
+	argList.reserve(end - begin + 1);
+	make_arg(cmd, argList);
+	for (auto it = begin; it != end; ++it) {
+		make_arg(*it, argList);
+	}
+
+	argV.reset(new char *[argList.size() + 1]);
+	char ** v = argV.get();
+	for (auto const& a : argList) {
+		*(v++) = a.get();
+	}
+	*v = nullptr;
+}
 }
 
 class process::impl
@@ -331,30 +356,7 @@ public:
 			err_.create();
 	}
 
-	void make_arg(native_string const& arg, std::vector<std::unique_ptr<native_string::value_type[]>> & argList)
-	{
-		std::unique_ptr<char[]> ret;
-		ret.reset(new char[arg.size() + 1]);
-		strcpy(ret.get(), arg.c_str());
-		argList.push_back(std::move(ret));
-	}
-
-	void get_argv(native_string const& cmd, std::vector<native_string> const& args, std::vector<std::unique_ptr<char[]>> & argList, std::unique_ptr<char *[]> & argV)
-	{
-		make_arg(cmd, argList);
-		for (auto const& a : args) {
-			make_arg(a, argList);
-		}
-
-		argV.reset(new char *[argList.size() + 1]);
-		char ** v = argV.get();
-		for (auto const& a : argList) {
-			*(v++) = a.get();
-		}
-		*v = nullptr;
-	}
-
-	bool spawn(native_string const& cmd, std::vector<native_string> const& args)
+	bool spawn(native_string const& cmd, std::vector<native_string>::const_iterator const& begin, std::vector<native_string>::const_iterator const& end)
 	{
 		if (pid_ != -1) {
 			return false;
@@ -366,9 +368,9 @@ public:
 
 		std::vector<std::unique_ptr<char[]>> argList;
 		std::unique_ptr<char *[]> argV;
-		get_argv(cmd, args, argList, argV);
+		get_argv(cmd, begin, end, argList, argV);
 
-		int pid = fork();
+		pid_t pid = fork();
 		if (pid < 0) {
 			return false;
 		}
@@ -476,7 +478,16 @@ process::~process()
 
 bool process::spawn(native_string const& cmd, std::vector<native_string> const& args)
 {
-	return impl_ ? impl_->spawn(cmd, args) : false;
+	return impl_ ? impl_->spawn(cmd, args.cbegin(), args.cend()) : false;
+}
+
+bool process::spawn(std::vector<native_string> const& command_with_args)
+{
+	if (command_with_args.empty()) {
+		return false;
+	}
+	auto begin = command_with_args.begin() + 1;
+	return impl_ ? impl_->spawn(command_with_args.front(), begin, command_with_args.end()) : false;
 }
 
 void process::kill()
@@ -496,4 +507,65 @@ bool process::write(char const* buffer, unsigned int len)
 	return impl_ ? impl_->write(buffer, len) : false;
 }
 
+bool spawn_detached_process(std::vector<native_string> const& cmd_with_args)
+{
+	if (cmd_with_args.empty()) {
+		return false;
+	}
+
+#ifdef FZ_WINDOWS
+	STARTUPINFO si{};
+	si.cb = sizeof(si);
+
+	auto begin = cmd_with_args.cbegin() + 1;
+	auto cmdline = get_cmd_line(cmd_with_args.front(), begin, cmd_with_args.cend());
+
+	PROCESS_INFORMATION pi{};
+
+	auto cmdline_buf = cmdline.data();
+
+	DWORD const flags = CREATE_UNICODE_ENVIRONMENT | CREATE_DEFAULT_ERROR_MODE | CREATE_NO_WINDOW;
+	BOOL res = CreateProcess(cmd_with_args.front().c_str(), cmdline_buf, nullptr, nullptr, TRUE, flags, nullptr, nullptr, &si, &pi);
+	if (!res) {
+		return false;
+	}
+
+	reset_handle(pi.hProcess);
+	reset_handle(pi.hThread);
+	return true;
+#else
+	if (cmd_with_args[0][0] != '/') {
+		return false;
+	}
+
+	std::vector<std::unique_ptr<char[]>> argList;
+	std::unique_ptr<char *[]> argV;
+	auto begin = cmd_with_args.cbegin() + 1;
+	get_argv(cmd_with_args.front(), begin, cmd_with_args.cend(), argList, argV);
+
+	pid_t const parent = getppid();
+	pid_t const ppgid = getpgid(parent);
+	pid_t pid = fork();
+	if (!pid) {
+		pid_t inner_pid = fork();
+		if (!inner_pid) {
+			// Change the process group ID of the new process so that terminating the outer process does not terminate the child
+			setpgid(0, ppgid);
+			execv(argV.get()[0], argV.get());
+			_exit(-1);
+		}
+		else {
+			_exit(0);
+		}
+	}
+	else {
+		int ret;
+		do {
+		} while ((ret = waitpid(pid, nullptr, 0)) == -1 && errno == EINTR);
+
+		return ret != -1;
+	}
+	return false;
+#endif
+}
 }
